<?php

/**
 * GooyaPay payment plugin
 *
 * @developer GooyaPay Technical Support
 * @publisher GooyaPay
 * @package RsForm Pro
 * @subpackage payment
 * @copyright (C) 2024 GooyaPay
 * @license http://www.gnu.org/licenses/gpl-2.0.html GPLv2 or later
 *
 * https://gooyapay.ir
 */

defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Http\Http;
use Joomla\CMS\Http\HttpFactory;
class plgSystemRSFPGooyapay extends JPlugin
{
  var $componentId = 4920;

  var $componentValue = 'gooyapay';

  /**
   * plgSystemRSFPGooyapay constructor.
   * @param $subject
   * @param $config
   * @param Http|null $http
   */
  public function __construct(&$subject, $config, Http $http = null)
  {
    $this->http = $http ?: HttpFactory::getHttp();
    parent::__construct($subject, $config);
    $this->newComponents = array(4920);
  }

  public function onRsformDefineHiddenComponents(&$hiddenComponents)
  {
    $hiddenComponents[] = $this->componentId;
  }


  public function onRsformBackendAfterShowComponents()
  {
    $this->rsfp_bk_onAfterShowComponents();
  }

  function rsfp_bk_onAfterShowComponents()
  {
    $lang = JFactory::getLanguage();
    $lang->load('plg_system_rsfpgooyapay');
    $formId = JRequest::getInt('formId');
    $link = "displayTemplate('" . $this->componentId . "')";
    if ($components = RSFormProHelper::componentExists($formId, $this->componentId))
      $link = "displayTemplate('" . $this->componentId . "', '" . $components[0] . "')";
    ?>
      <li class="rsform_navtitle">درگاه گویاپـــی</li>
      <li><a href="javascript: void(0);" onclick="<?php echo $link; ?>;return false;" id="rsfpc<?php echo $this->componentId; ?>"><span id="GOOYAPAY">افزودن درگاه گویاپـــی</span></a></li>
    <?php
  }

  public function onRsformGetPayment(&$items, $formId)
  {
    $this->rsfp_getPayment($items, $formId);
  }

  function rsfp_getPayment(&$items, $formId)
  {
    if ($components = RSFormProHelper::componentExists($formId, $this->componentId)) {
      $data = RSFormProHelper::getComponentProperties($components[0]);
      $item = new stdClass();
      $item->value = $this->componentValue;
      $item->text = $data['LABEL'] . 'پرداخت از طریق گویاپـــی';

      //JURI::root(true).'/plugins/system/rsfpgooyapay/assets/images/logo.png
      $items[] = $item;
    }
  }

  public function onRsformBackendAfterCreateComponentPreview($args = array())
  {
    $this->rsfp_bk_onAfterCreateComponentPreview($args);
  }

  function rsfp_bk_onAfterCreateComponentPreview($args = array())
  {
    if ($args['ComponentTypeName'] == 'gooyapay') {
      $args['out'] = '<td>&nbsp;gooyapay</td>';
      $args['out'] .= '<td><img style="max-width:100px; max-height:35px;" src=' . JURI::root(true) . '/plugins/system/rsfpgooyapay/assets/images/logo.png />' . $args['data']['LABEL'] . '</td>';
    }
  }

  public function onRsformBackendAfterShowConfigurationTabs($tabs)
  {
    $this->rsfp_bk_onAfterShowConfigurationTabs($tabs);
  }

  function rsfp_bk_onAfterShowConfigurationTabs($tabs)
  {
    $lang = JFactory::getLanguage();
    $lang->load('plg_system_rsfpgooyapay');
    $tabs->addTitle('تنظیمات درگاه گویاپـــی', 'form-TRANGELGOOYAPAY');
    $tabs->addContent($this->gooyapayConfigurationScreen());
  }

  public function onRsformDoPayment($payValue, $formId, $SubmissionId, $price, $products, $code)
  {
    $this->rsfp_doPayment($payValue, $formId, $SubmissionId, $price, $products, $code);
  }

  function rsfp_doPayment($payValue, $formId, $SubmissionId, $price, $products, $code)
  {
    $components = RSFormProHelper::componentExists($formId, $this->componentId);
    $data = RSFormProHelper::getComponentProperties($components[0]);
    $app = JFactory::getApplication();

    if ($data['TOTAL'] == 'YES') {
      $price = (int)$_POST['form']['rsfp_Total'];
    } elseif ($data['TOTAL'] == 'NO') {
      if ($data['FIELDNAME'] == 'Select the desired field') {
        $msg = 'فیلدی برای قیمت انتخاب نشده است.';
        $link = JURI::root() . 'index.php?option=com_rsform&formId=' . $formId;
        $app->redirect($link, '<h2>' . $msg . '</h2>', $msgType = 'Error');
      }
      $price = $_POST['form'][$data['FIELDNAME']];
    }

    if (is_array($price))
      $price = (int)array_sum($price);

    if (!$price) {
      $msg = 'مبلغی وارد نشده است';
      $link = JURI::root() . 'index.php?option=com_rsform&formId=' . $formId;
      $app->redirect($link, '<h2>' . $msg . '</h2>', $msgType = 'Error');
    }

    $currency = RSFormProHelper::getConfig('gooyapay.currency');
    $price = $this->rsfp_gooyapay_get_amount($price, $currency);

    // execute only for our plugin
    if ($payValue != $this->componentValue) return;
    $tax = RSFormProHelper::getConfig('gooyapay.tax.value');
    if ($tax)
      $nPrice = round($tax, 0) + round($price, 0);
    else
      $nPrice = round($price, 0);

    if ($nPrice >= 2000) {
      $merchant_key = RSFormProHelper::getConfig('gooyapay.merchant');
      $amount = $nPrice;
      $desc = 'پرداخت سفارش شماره: ' . $formId;
      $callback = JURI::root() . 'index.php?option=com_rsform&task=plugin&plugin_task=gooyapay.notify&code=' . $code;
      if (empty($amount)) {
        $msg = 'واحد پول انتخاب شده پشتیبانی نمی شود.';
        $link = JURI::root() . 'index.php?option=com_rsform&formId=' . $formId;
        $app->redirect($link, '<h2>' . $msg . '</h2>', $msgType = 'Error');
      }

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, 'https://gooyapay.ir/webservice/rest/PaymentRequest');
		curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode(array(
				'MerchantID' 	=> $merchant_key,
				'Amount' 		=> $amount,
				'InvoiceID' 	=> $formId,
				'Description' 	=> $desc,
				'CallbackURL' 	=> $callback
			)));
		curl_setopt($ch, CURLOPT_HTTPHEADER, array(
				'Content-Type: application/json',
				'Accept: application/json'
			));
		curl_setopt($ch, CURLOPT_TIMEOUT, 30);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		$response = curl_exec($ch);
		$error = curl_error($ch);
		curl_close($ch);
		$result = json_decode($response);
		if (isset($result->Status) && $result->Status == 100)
		{
			$db = JFactory::getDBO();
			$sql = 'INSERT INTO `#__rsform_submission_values` (FormId, SubmissionId, FieldName,FieldValue) VALUES (' . $formId . ',' . $SubmissionId . ',"gooyapay_id","' . $result->Authority . '")';
			$db->setQuery($sql);
			$db->execute();
			$app->redirect($result->PaymentUrl);
			header('Location: ' . $result->PaymentUrl);
			exit;
		}
		else
		{
			if (isset($result->Message) && $result->Message != '')
			{
				$error = 'Error : ' . $result->Message;
			}
			elseif ($error)
			{
				$error = 'Connecting Error : ' . $error;
			}
			else
			{
				$error = 'UnExpected Error in connecting to web service';
			}
			$msg = 'خطا هنگام ایجاد تراکنش . شرح خطا: ' . $error;
			$link = JURI::root() . 'index.php?option=com_rsform&formId=' . $formId;
			$this->updateAfterEvent($formId, $SubmissionId, $msg);
			$app->redirect($link, '<h2>' . $msg . '</h2>', $msgType = 'Error');
		}
    } else {
      $msg = 'مبلغ وارد شده کمتر از 20.000 ریال می باشد';
      $link = JURI::root() . 'index.php?option=com_rsform&formId=' . $formId;
      $app->redirect($link, '<h2>' . $msg . '</h2>', $msgType = 'Error');
    }
  }

  public function onRsformFrontendSwitchTasks()
  {
    $this->rsfp_f_onSwitchTasks();
  }

  function rsfp_f_onSwitchTasks()
  {
    if (JRequest::getVar('plugin_task') == 'gooyapay.notify') {
      $app = JFactory::getApplication();
      $jinput = $app->input;

      $track_id = $_POST['Authority'];
      $formId = $_POST['InvoiceID'];
      $pid = $_POST['Authority'];
      $pOrderId = $_POST['InvoiceID'];
      $status = $_POST['PaymentStatus'];

      $code = $jinput->get->get('code', '', 'STRING');
      $db = JFactory::getDBO();
      $db->setQuery("SELECT SubmissionId FROM #__rsform_submissions s WHERE s.FormId='" . $formId . "' AND MD5(CONCAT(s.SubmissionId,s.DateSubmitted)) = '" . $db->escape($code) . "'");
      $SubmissionId = $db->loadResult();
      $components = RSFormProHelper::componentExists($formId, $this->componentId);
      $data = RSFormProHelper::getComponentProperties($components[0]);
      $app = JFactory::getApplication();

      if ($data['TOTAL'] == 'YES') {
        $fieldname="rsfp_Total";
      } elseif ($data['TOTAL'] == 'NO') {
        $fieldname=$data['FIELDNAME'];
      }

      $price = round($this::getPayerPrice($formId, $SubmissionId,$fieldname), 0);

      //convert to currency
      $currency = RSFormProHelper::getConfig('gooyapay.currency');
      $price = $this->rsfp_gooyapay_get_amount($price, $currency);

      $order_id = $formId;
      if (!empty($pid) && !empty($pOrderId) && $pOrderId == $order_id) {

        //in waiting confirm
        if ($status == 'OK') {
          $merchant_key = RSFormProHelper::getConfig('gooyapay.merchant');

			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, 'https://gooyapay.ir/webservice/rest/PaymentVerification');
			curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode(array(
					'MerchantID' 	=> $merchant_key,
					'Amount' 		=> $price,
					'Authority' 	=> $track_id
				)));
			curl_setopt($ch, CURLOPT_HTTPHEADER, array(
					'Content-Type: application/json',
					'Accept: application/json'
				));
			curl_setopt($ch, CURLOPT_TIMEOUT, 30);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
			$response = curl_exec($ch);
			curl_close($ch);
			$result = json_decode($response);
			if (isset($result->Status) && $result->Status == 100)
			{
				//check double spending
				$db = JFactory::getDBO();
				$sql = 'SElECT FieldValue FROM ' . "#__rsform_submission_values" . '  WHERE FieldName="gooyapay_id" AND FormId=' . $formId . ' AND SubmissionId = ' . $SubmissionId;
				$db->setQuery($sql);
				$db->execute();
				$exist = $db->loadObjectList();
				var_dump($exist);
				$exist = count($exist);

				if (!$exist) {
					$msg = 'تراکنش تکراری بوده و قبلا تعیین وضعیت شده';
					$msg = $this->gooyapay_get_failed_message($track_id, $order_id, $msg);
					$this->updateAfterEvent($formId, $SubmissionId, $msg);
					$link = JURI::root() . 'index.php?option=com_rsform&formId=' . $formId;
					$app->redirect($link, '<h2>' . $msg . '</h2>', $msgType = 'Error');
				}

				//save gooyapay payment data in db
				$sql = 'INSERT INTO `#__rsform_submission_values` (FormId, SubmissionId, FieldName,FieldValue) VALUES (' . $formId . ',' . $SubmissionId . ',"gooyapay_track_id","' . $result->RefID . '")';
				$db->setQuery($sql);
				$db->execute();
				$sql = 'INSERT INTO `#__rsform_submission_values` (FormId, SubmissionId, FieldName,FieldValue) VALUES (' . $formId . ',' . $SubmissionId . ',"gooyapay_card_no","' . $result->MaskCardNumber . '")';
				$db->setQuery($sql);
				$db->execute();

				$mainframe = JFactory::getApplication();
				$mainframe->triggerEvent('rsfp_afterConfirmPayment', array($SubmissionId));
				$msgForSaveDataTDataBase = "کد پیگیری :  $result->RefID " . "شماره کارت :  $result->MaskCardNumber ";
				$this->updateAfterEvent($formId, $SubmissionId, $msgForSaveDataTDataBase);
				$msg = $this->gooyapay_get_success_message($result->RefID, $order_id);
				$link = JURI::root() . 'index.php?option=com_rsform&formId=' . $formId;
				$app->redirect($link, '<h2>' . $msg . '</h2>', $msgType = 'success');
			}
			else
			{
				if (isset($result->Message) && $result->Message != '')
				{
					$error = 'Error : ' . $result->Message;
				}
				elseif ($error)
				{
					$error = 'Connecting Error : ' . $error;
				}
				else
				{
					$error = 'UnExpected Error in connecting to web service';
				}
				$msg = sprintf('خطا هنگام بررسی وضعیت تراکنش - شرح خطا: %s ', $error);
				$msg = $this->gooyapay_get_failed_message($track_id, $order_id, $msg);
				$this->updateAfterEvent($formId, $SubmissionId, $msg);
				$link = JURI::root() . 'index.php?option=com_rsform&formId=' . $formId;
				$app->redirect($link, '<h2>' . $msg . '</h2>', $msgType = 'Error');
			}
        } else {
          //save pay failed pay message(payment api)
          $msg = 'تراکنش توسط کاربر لغو شد';
          $msg = $this->gooyapay_get_failed_message($track_id, $order_id, $msg);
          $this->updateAfterEvent($formId, $SubmissionId, $msg);
          $link = JURI::root() . 'index.php?option=com_rsform&formId=' . $formId;
          $app->redirect($link, '<h2>' . $msg . '</h2>', $msgType = 'Error');
        }

      } else {

        $msg = 'اطلاعات دریافتی همخوانی ندارند';
        $msg = $this->gooyapay_get_failed_message($track_id, $order_id, $msg);
        $this->updateAfterEvent($formId, $SubmissionId, $msg);
        $link = JURI::root() . 'index.php?option=com_rsform&formId=' . $formId;
        $app->redirect($link, '<h2>' . $msg . '</h2>', $msgType = 'Error');
      }

    } else {
      return NULL;
    }
  }

  /**
   * @return false|string
   */
  function gooyapayConfigurationScreen()
  {
    ob_start();
    ?>
      <div id="page-gooyapay" class="com-rsform-css-fix">
          <table class="admintable">
              <tr>
                  <td width="200" style="width: 200px;" align="right" class="key"><label for="merchant">Merchant Code</label></td>
                  <td><input type="text" name="rsformConfig[gooyapay.merchant]"
                             value="<?php echo RSFormProHelper::htmlEscape(RSFormProHelper::getConfig('gooyapay.merchant')); ?>"
                             size="100" maxlength="64"></td>
              </tr>
              <tr>
                  <td width="200" style="width: 200px;" align="right" class="key">
                      <label><?php echo 'currency'; ?></label></td>
                  <td>
                    <?php
                    echo(RSFormProHelper::htmlEscape(RSFormProHelper::getConfig('gooyapay.currency')));

                    ?>
                      <select name="rsformConfig[gooyapay.currency]">
                          <option value="RIAL"<?php echo(RSFormProHelper::htmlEscape(RSFormProHelper::getConfig('gooyapay.currency')) == 'RIAL' ? 'selected="selected"' : ""); ?>>
                              ریال
                          </option>
                          <option value="IRT"<?php echo(RSFormProHelper::htmlEscape(RSFormProHelper::getConfig('gooyapay.currency')) == 'IRT' ? 'selected="selected"' : ""); ?>>
                              تومان
                          </option>
                      </select>
                  </td>
              </tr>
              <tr>
                  <td width="200" style="width: 200px;" align="right" class="key">
                      <label><?php echo 'پیام پرداخت موفق'; ?></label></td>
                  <td><textarea
                              name="rsformConfig[gooyapay.success_massage]"><?php echo(!empty(RSFormProHelper::htmlEscape(RSFormProHelper::getConfig('gooyapay.success_massage'))) ? RSFormProHelper::htmlEscape(RSFormProHelper::getConfig('gooyapay.success_massage')) : "پرداخت شما با موفقیت انجام شد. کد رهگیری: {track_id}"); ?></textarea><br>متن
                      پیامی که می خواهید بعد از پرداخت موفق به کاربر نمایش دهید را وارد کنید. همچنین می توانید از شورت
                      کدهای {order_id} برای نمایش شماره سفارش و {track_id} برای نمایش کد رهگیری گویاپـــی استفاده
                      نمایید.
                  </td>
              </tr>
              <tr>
                  <td width="200" style="width: 200px;" align="right" class="key">
                      <label><?php echo 'پیام پرداخت ناموفق'; ?></label></td>
                  <td><textarea
                              name="rsformConfig[gooyapay.failed_massage]"><?php echo(!empty(RSFormProHelper::htmlEscape(RSFormProHelper::getConfig('gooyapay.failed_massage'))) ? RSFormProHelper::htmlEscape(RSFormProHelper::getConfig('gooyapay.failed_massage')) : "پرداخت شما ناموفق بوده است. لطفا مجددا تلاش نمایید یا در صورت بروز اشکال با مدیر سایت تماس بگیرید."); ?></textarea><br>متن
                      پیامی که می خواهید بعد از پرداخت ناموفق به کاربر نمایش دهید را وارد کنید. همچنین می توانید از
                      شورت کدهای {order_id} برای نمایش شماره سفارش و {track_id} برای نمایش کد رهگیری گویاپـــی استفاده
                      نمایید.
                  </td>
              </tr>
          </table>
      </div>

    <?php

    $contents = ob_get_contents();
    ob_end_clean();
    return $contents;
  }

  /**
   * @param $formId
   * @param $SubmissionId
   * @return mixed
   */
  function getPayerMobile($formId, $SubmissionId)
  {
    $db = JFactory::getDbo();
    $query = $db->getQuery(true);
    $query->select('FieldValue')->from($db->qn('#__rsform_submission_values'));
    $query->where($db->qn('FormId') . ' = ' . $db->q($formId) . ' AND ' . $db->qn('SubmissionId') . ' = ' . $db->q($SubmissionId) . ' AND ' . $db->qn('FieldName') . ' = ' . $db->q('mobile'));
    $db->setQuery((string)$query);
    $result = $db->loadResult();
    return $result;
  }

  /**
   * @param $formId
   * @param $SubmissionId
   * @param $fieldName
   * @return int
   */
  function getPayerPrice($formId, $SubmissionId, $fieldName)
  {
    $db = JFactory::getDbo();
    $query = $db->getQuery(true);
    $query->select('FieldValue')->from($db->qn('#__rsform_submission_values'));
    $query->where($db->qn('FormId') . ' = ' . $db->q($formId) . ' AND ' . $db->qn('SubmissionId') . ' = ' . $db->q($SubmissionId) . ' AND ' . $db->qn('FieldName') . ' = ' . $db->q($fieldName));
    $db->setQuery((string)$query);
    $result = $db->loadResult();

    return (int)$result;
  }

  /**
   * @param $track_id
   * @param $order_id
   * @param null $msgNumber
   * @return string
   */
  public function gooyapay_get_failed_message($track_id, $order_id, $msg = null)
  {
    //get defult massege
    $failedMassage = RSFormProHelper::getConfig('gooyapay.failed_massage');
    return str_replace(["{track_id}", "{order_id}"], [$track_id, $order_id], $failedMassage) . "<br>" . $msg;
  }

  /**
   * @param $track_id
   * @param $order_id
   * @return string|string[]
   */
  public function gooyapay_get_success_message($track_id, $order_id)
  {
    //get defult success massage
    $successMassage = RSFormProHelper::getConfig('gooyapay.success_massage');
    return str_replace(["{track_id}", "{order_id}"], [$track_id, $order_id], $successMassage);
  }

  /**
   * @param $amount
   * @param $currency
   * @return float|int
   */
  function rsfp_gooyapay_get_amount($amount, $currency)
  {
    switch (strtolower($currency)) {
      case strtolower('IRR'):
      case strtolower('RIAL'):
        return intval(ceil($amount / 10));

      case strtolower('تومان ایران'):
      case strtolower('تومان'):
      case strtolower('IRT'):
      case strtolower('Iranian_TOMAN'):
      case strtolower('Iran_TOMAN'):
      case strtolower('Iranian-TOMAN'):
      case strtolower('Iran-TOMAN'):
      case strtolower('TOMAN'):
      case strtolower('Iran TOMAN'):
      case strtolower('Iranian TOMAN'):
        return intval(ceil($amount));

      case strtolower('IRHR'):
        return intval(ceil($amount * 100));

      case strtolower('IRHT'):
        return intval(ceil($amount * 1000));

      default:
        return 0;
    }
  }

  /**
   * @param $formId
   * @param $SubmissionId
   * @param $msg
   * @return bool
   */
  public function updateAfterEvent($formId, $SubmissionId, $msg)
  {
    if (!$SubmissionId) {
      return false;
    }
    $db = JFactory::getDBO();
    $msg = "gooyapay: $msg";
    $db->setQuery("UPDATE #__rsform_submission_values sv SET sv.FieldValue=1 WHERE sv.FieldName='_STATUS' AND sv.FormId='" . $formId . "' AND sv.SubmissionId = '" . $SubmissionId . "'");
    $db->execute();
    $db->setQuery("UPDATE #__rsform_submission_values sv SET sv.FieldValue='" . $msg . "'  WHERE sv.FieldValue='gooyapay' AND sv.FormId='" . $formId . "' AND sv.SubmissionId = '" . $SubmissionId . "'");
    $db->execute();
  }

}
